# GitLab CI C/C++ Example

Please refer to the [official documentation](https://docs.gitlab.com/ce/ci/yaml/README.html)
to view all configuration options available.

[![pipeline status](https://gitlab.fi.muni.cz/unix/ci-examples/badges/c-cmake/pipeline.svg)](https://gitlab.fi.muni.cz/unix/ci-examples/commits/c-cmake)

This project contains a simple C library that provies a `factorial` function.
Tests are written using [`Catch`](https://github.com/catchorg/Catch2) framework
and are located inside `tests` directory.
The `CMakeLists.txt` defines two goals, a library `libfact.a` and an
executable `fact_tests`.

Let's have a look at `.gitlab-ci.yml`:

```yml
image: amardikian/clang-3.8
```

This line informs the runner that we want to use [amardikian/clang-3.8](https://hub.docker.com/r/amardikian/clang-3.8/)
image which contains CLang as well as CMake.

```yml
stages:
  - build
  - test
```

Our pipeline has two stages, `build` and `test`. You can learn more
about stages [here](https://docs.gitlab.com/ce/ci/yaml/README.html#stages).

Next two sections describe the details of each stage:

```yml
build:
  stage: build
```

The second line seems superfluous, but in general the stages do not need
to be called `build` or `test` and there can be more than one.
The `stage: build` declaration merely informs the runner that this stage
is actually part of the build *phase*.

```yml
  tags:
    - shared-fi
```

`gitlab-ci.fi.muni.cz` will only run jobs that have tag `shared-fi`.

```yml
  variables:
    CC: clang
    CXX: clang++
```

This configuration sets environment variables `CC` and `CXX` to
`clang` and `clang++` respectively.

```yml
  script:
    - cmake -Bbuild -H.
    - make -Cbuild -j
```

Finally, the `script` section contains a list of commands that shall be
performed inside the repository once the runner starts.
These *should* be executed inside a single shell instance so that
`cd` and similar commands should work as expected, but unfortunately
this is not specified in the official documentation.

```yml
  artifacts:
    untracked: true
```

When using separate pipeline stages where one stage requires files created
in some previous stage, they need to be passed as *artifacts*.
Here, the `test` phase needs compiled library and an executable.
The `untracked: true` configuration simply marks all files **not** tracked
by git as artifacts.

This concludes the `build` stage, so let's have a look at the `test` stage.

```yml
test:
  stage: test
  tags:
    - shared-fi
  script:
    - build/fact_test -r compact
```

Similar to the `build` stage, we declare that `test` stage is a part
of the `test` *phase* and again, we need to mark the stage with the `shared-fi` tag.
The script part simply runs compiled tests.

```yml
  dependencies:
    - build
```

This declaration states that the `test` stage should run after a successful
`build` stage, using artifacts it creates.
